<?php
/**
  * @version     3.0 +
  * @package     Open Source Security Suite
  * @author      Open Source Excellence (R) {@link  http://www.opensource-excellence.com}
  * @author      Created on 17-May-2011
  * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
  *
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *  @Copyright Copyright (C) 2010- Open Source Excellence (R)
*/
defined('_JEXEC') or die("Direct Access Not Allowed");
class oseInstallerHelper
{
	var $backendPath;
	var $frontendPath;
	var $cpuFile;
	var $successStatus;
	var $failedStatus;
	var $notApplicable;
	var $totalStep;
	var $pageTitle;
	var $verifier;
	var $dbhelper;
	var $template;
	var $component;
	var $frontendCPUPath;
	function __construct()
	{
		$this->component= OSE_COMP_FOLDERNAME;
		$this->com_title= OSE_COMP_TITLE;
		$this->cpuFile = 'cpuATH.zip';
		$this->backendPath= OSE_COMP_BACKPATH.DS;
		$this->frontendPath= OSE_COMP_FRONTPATH.DS;
		$this->frontendCPUPath= OSE_CPU_FRONTPATH.DS;
		$this->totalStep= 5;
		$this->setStyleVariables(); 
		$this->importCoreClass(); 
		$this->template= new oseInstallerTemplate();
		$this->verifier= new oseInstallerVerifier();
	}
	private function importCoreClass()
	{
		jimport('joomla.application.component.controller');
		jimport('joomla.application.component.model');
		jimport('joomla.installer.installer');
		jimport('joomla.installer.helper');
		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.archive');
		jimport('joomla.filesystem.path');
		require_once(OSE_COMP_BACKPATH.DS.'installer.template.php');
	}
	private function setStyleVariables()
	{
		$document = JFactory :: getDocument();
		$document->addScript(JURI :: root() . 'media/system/js/mootools-core.js');
		$this->successStatus= '<div style="float:left;">.....&nbsp;</div><div style="color:#009900;">'.JText :: _('Installation completed').'</div><div style="clear:both;"></div>';
		$this->failedStatus= '<div style="float:left;">.....&nbsp;</div><div style="color:red;">'.JText :: _('Installation failed').'</div><div style="clear:both;"></div>';
		$this->notApplicable= '<div style="float:left;">.....&nbsp;</div><div>'.JText :: _('Installation not applicable').'</div><div style="clear:both;"></div>';
	}
	function install()
	{
		//check php version
		$installedPhpVersion= floatval(phpversion());
		$supportedPhpVersion= 5.2;
		$step= JRequest :: getVar('step', '', 'post');
		$helper= new oseInstallerHelper;
		if($installedPhpVersion < $supportedPhpVersion)
		{
			$html= oseInstallerHelper :: getErrorMessage(101, $installedPhpVersion);
			$status= false;
			$nextstep= 0;
			$title= JText :: _('OSE Installer for').' '.$this->com_title;
			$install= 1;
			$substep= 0;
		}
		else
		{
			if(!empty($step))
			{
				$progress= $helper->installSteps($step);
				$html= $progress->message;
				$status= $progress->status;
				$nextstep= $progress->step;
				$title= $progress->title;
				$install= $progress->install;
				$substep= isset($progress->substep) ? $progress->substep : 0;
			}
			else
			{
				$nextstep= 1;
				$verifier= new oseInstallerVerifier();
				$imageTest= $verifier->testImage();
				$template= new oseInstallerTemplate();
				$html= $template->getHTML('welcome', $imageTest);
				$status= true;
				$title= JText :: _('OSE Installer for').' '.$this->com_title;
				$install= 1;
				$substep= 0;
			}
		}
		$this->template->cInstallDraw($html, $nextstep, $title, $status, $install, $substep);
		return;
	}
	public function installSteps($step= 1)
	{
		switch($step)
		{
			case 1 :
				//check requirement
				$status= $this->checkRequirement(2);
				break;
			case 2 :
				//install backend system
				$status= $this->installBackend(3);
				break;
			case 3 :
				//install ajax system
				$status= $this->installCOMCPU(4);
				break;
			case 4 :
				//install frontend system
				$status= $this->installFrontend(5);
				break;
			case 5 :
				//install template
				$status= $this->prepareDatabase(6);
				break;
			case 6 :
			case 'UPDATE_DB' :
				//prepare database
				$status= $this->updateDatabase(7);
				break;
			case 7 :
				$status= $this->installPlugin(8);
				break;
			case 8 :
				$status= $this->installViews(9);
				break;
			case 9 :
				$status= $this->clearInstallation(100);
				break;
			case 100 :
				//show success message
				$status= $this->installationComplete(0);
				break;
			default :
				$status= new stdClass();
				$status->message= $this->getErrorMessage(0, '0a');
				$status->step= '-99';
				$status->title= JText :: _('OSE INSTALLER');
				$status->install= 1;
				break;
		}
		return $status;
	}
	private function checkRequirement($step)
	{
		$status= true;
		$this->pageTitle= JText :: _('Checking Requirements');
		$html= '';
		$html .= '<div style="width:300px; float:left;">'.JText :: _('BACKEND ARCHIVE').'</div>';
		if(!$this->verifier->checkFileExist($this->backendPath.'admin.zip'))
		{
			$html .= $this->failedStatus;
			$status= false;
			$errorCode= '1a';
		}
		else
		{
			$html .= $this->successStatus;
		}
		$html .= '<div style="width:300px; float:left;">'.JText :: _('OSE CPU Backend ARCHIVE').'</div>';
		if(!$this->verifier->checkFileExist($this->backendPath.'com_cpu_admin.zip'))
		{
			$html .= $this->failedStatus;
			$status= false;
			$errorCode= '1b';
		}
		else
		{
			$html .= $this->successStatus;
		}
		$html .= '<div style="width:300px; float:left;">'.JText :: _('OSE CPU FRONTEND ARCHIVE').'</div>';
		if(!$this->verifier->checkFileExist($this->backendPath.'com_cpu_site.zip'))
		{
			$html .= $this->failedStatus;
			$status= false;
			$errorCode= '1b';
		}
		else
		{
			$html .= $this->successStatus;
		}
		$html .= '<div style="width:300px; float:left;">'.JText :: _('COMPONENT CPU ARCHIVE for OSE CPU').'</div>';
		if(!$this->verifier->checkFileExist($this->backendPath.$this->cpuFile))
		{
			$html .= $this->failedStatus;
			$status= false;
			$errorCode= '1b';
		}
		else
		{
			$html .= $this->successStatus;
		}

		if($status)
		{
			$autoSubmit= $this->getAutoSubmitFunction();
			//$form = $this->getInstallForm(2);
			$message= $autoSubmit.$html;
		}
		else
		{
			$errorMsg= $this->getErrorMessage(1, $errorCode);
			$message= $html.$errorMsg;
			$step= $step -1;
		}
		$drawdata= new stdClass();
		$drawdata->message= $message;
		$drawdata->status= $status;
		$drawdata->step= $step;
		$drawdata->title= JText :: _('OSE CHECKING REQUIREMENT');
		$drawdata->install= 1;
		return $drawdata;
	}
	private function getAutoSubmitFunction()
	{
		ob_start();
		?>
		<script type="text/javascript">
		var i=3;
		function countDown()
		{
			if(i >= 0)
			{
				document.getElementById("timer").innerHTML = i;
				i = i-1;
				var c = window.setTimeout("countDown()", 1000);
			}
			else
			{
				document.getElementById("div-button-next").removeAttribute("onclick");
				document.getElementById("input-button-next").setAttribute("disabled","disabled");
				document.forms["installform"].submit();
			}
		}
		window.addEvent('domready', function() {
			countDown();
		});
		</script>
		<?php
		$autoSubmit= ob_get_contents();
		@ ob_end_clean();
		return $autoSubmit;
	}
	private function installBackend($step)
	{
		$html= '';
		$html .= '<div style="width:300px; float:left;">'.JText :: _('OSE BACKEND INSTALLATION').'</div>';
		$zip= $this->backendPath.'admin.zip';
		$destination= $this->backendPath;
		if($this->extractArchive($zip, $destination))
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$errorMsg= $this->getErrorMessage($step, $step);
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$html .= '<div style="width:300px; float:left;">'.JText :: _('English language file installation').'</div>';
		if($this->installLanguage('back'))
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$errorMsg= $this->getErrorMessage(4, '4');
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$drawdata= new stdClass();
		$drawdata->message= $message;
		$drawdata->status= $status;
		$drawdata->step= $step;
		$drawdata->title= JText :: _('OSE BACKEND INSTALLATION');
		$drawdata->install= 1;
		return $drawdata;
	}
	private function installCOMCPU($step)
	{
		$html= '';
		$html .= '<div style="width:300px; float:left;">'.JText :: _('OSE CPU BACKEND INSTALLATION').'</div>';
		$zip= $this->backendPath.'com_cpu_admin.zip';
		$destination= JPATH_ADMINISTRATOR.DS.'components'.DS.'com_ose_cpu'.DS;
		if($this->extractArchive($zip, $destination))
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$errorMsg= $this->getErrorMessage(2, '2');
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');
		if (JFile::exists($this->frontendCPUPath.'extjs'.DS.'init'))
		{
			JFile::delete($this->frontendCPUPath.'extjs'.DS.'init');
		}
		if (JFolder::exists($this->frontendCPUPath.'extjs'.DS.'init'))
		{
			JFolder::delete($this->frontendCPUPath.'extjs'.DS.'init');
		}
		$html .= '<div style="width:300px; float:left;">'.JText :: _('OSE CPU FRONTEND INSTALLATION').'</div>';
		$zip= $this->backendPath.'com_cpu_site.zip';
		$destination= JPATH_SITE.DS.'components'.DS.'com_ose_cpu'.DS;
		if($this->extractArchive($zip, $destination))
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$errorMsg= $this->getErrorMessage(2, '2');
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$zip= $this->backendPath.$this->cpuFile;
		$destination= JPATH_ADMINISTRATOR.DS.'components'.DS.'com_ose_cpu'.DS;
		$html .= '<div style="width:300px; float:left;">'.JText :: _('OSE CPU Extended INSTALLATION').'</div>';
		if($this->extractArchive($zip, $destination))
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$errorMsg= $this->getErrorMessage(2, '2');
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$drawdata= new stdClass();
		$drawdata->message= $message;
		$drawdata->status= $status;
		$drawdata->step= $step;
		$drawdata->title= JText :: _('OSE INSTALLING Central Processing Units');
		$drawdata->install= 1;
		return $drawdata;
	}
	private function installFrontend($step)
	{
		$html= '';
		$html .= '<div style="width:300px; float:left;">'.JText :: _('OSE Scanning File INSTALLATION').'</div>';
		$src = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_ose_antihacker'.DS.'scan'.DS.'scan.php';
		$dest = JPATH_ADMINISTRATOR.DS.'scan.php';
		if(JFile::copy($src, $dest))
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$errorMsg= $this->getErrorMessage(2, '2');
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$html .= '<div style="width:300px; float:left;">'.JText :: _('OSE Definition File INSTALLATION').'</div>';
		$src = JPATH_ADMINISTRATOR.DS.'components'.DS.'com_ose_antihacker'.DS.'scan'.DS.'ahsdefines.php';
		$dest = JPATH_ADMINISTRATOR.DS.'ahsdefines.php';
		if(JFile::copy($src, $dest))
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$errorMsg= $this->getErrorMessage(2, '2');
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$drawdata= new stdClass();
		$drawdata->message= $message;
		$drawdata->status= $status;
		$drawdata->step= $step;
		$drawdata->title= JText :: _('OSE INSTALLING Scanning Files');
		$drawdata->install= 1;
		return $drawdata;
	}
	private function installPlugin($step)
	{
		$html= '';
		$html .= '<div style="width:100px; float:left;">'.JText :: _('No Plugins to install for this component').'</div>';
		$result= null;
		$db= JFactory :: getDBO();

		$result= true;
		$viewhtml= '';

		if($result == true)
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			//$form = $this->getInstallForm(5);
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$html .= $viewhtml;
			$errorMsg= $this->getErrorMessage($step, $step);
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$drawdata= new stdClass();
		$drawdata->message= $message;
		$drawdata->status= true;
		$drawdata->step= $step;
		$drawdata->title= JText :: _('CREATING VIEWS');
		$drawdata->install= 1;
		return $drawdata;
	}
	private function prepareDatabase($step)
	{
		$html= '';
		$html .= '<div style="width:300px; float:left;">'.JText :: _('Creating Database').'</div>';
		$queryResult= $this->installSQL();
		if($queryResult == true)
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			//$form = $this->getInstallForm(7);
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$errorMsg= $this->getErrorMessage(6, $queryResult);
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$drawdata= new stdClass();
		$drawdata->message= $message;
		$drawdata->status= $status;
		$drawdata->step= $step;
		$drawdata->title= JText :: _('PREPARING DATABASE');
		$drawdata->install= 1;
		return $drawdata;
	}
	private function UpdateDatabase($step)
	{
		$html= '';
		$html .= '<div style="width:300px; float:left;">'.JText :: _('Fix Database Integrity').'</div>';
		$queryResult= $this->fixIntegrity();
		if($queryResult == true)
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			//$form = $this->getInstallForm(7);
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$errorMsg= $this->getErrorMessage(7, $queryResult);
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$stage = $this->getGeoIPStage();
		$html .= '<div style="width:300px; float:left;">'.JText :: _('Installing GeoIPLocation Database'). ' Dataset ->'. $stage.'</div>';
		
		if ($stage<=6)
		{	
			$queryResult= $this->installGeoIPDB($stage);
			if($queryResult == true)
			{
				$html .= $this->successStatus;
				$autoSubmit= $this->getAutoSubmitFunction();
				//$form = $this->getInstallForm(7);
				$message= $autoSubmit.$html;
				$status= true;
			}
			else
			{
				$html .= $this->failedStatus;
				$errorMsg= $this->getErrorMessage(7, $queryResult);
				$message= $html.$errorMsg;
				$status= false;
				$step= $step -1;
			}
		}
		if ($stage <= 6)
		{
			$drawdata= new stdClass();
			$drawdata->message= $message;
			$drawdata->status= $status;
			$drawdata->step= $step-1;
			$drawdata->title= JText :: _('Installing GeoIP Data'). 'Dataset ->'. $stage;
			$drawdata->install= 1;
		}
		else
		{		
			$drawdata= new stdClass();
			$drawdata->message= $message;
			$drawdata->status= $status;
			$drawdata->step= $step;
			$drawdata->title= JText :: _('Fixing Database Integrity');
			$drawdata->install= 1;
		}
		return $drawdata;
	}
	private function getGeoIPStage()
	{
		$db = JFactory::getDBO();
		$query = "SELECT COUNT(id) FROM #__ose_app_geoip ";
		$db->setQuery($query);
		$result = $db->loadResult();
		$return = ceil ($result / 25000); 
		return $return; 
	}
	private function installGeoIPDB($stage)
	{
		$db = JFactory::getDBO();
		$file = JPATH_COMPONENT.DS.'sql'.DS.'osegeoip'.$stage.'.sql';
		$data = JFile::read($file);
		$queries = $this->_splitQueries($data);
		foreach ($queries as $query)
		{	
				$db->setQuery($query);
				if(!$db->query()) {
					echo JText :: _('Unable to insert GeoIP record').'-'.$stage;
					echo $db->getErrorMsg();
					return false;
				}
		}
		unset($queries); 
		unset($data);
		return true; 
	}
	private function _splitQueries($sql)
	{
		// Initialise variables.
		$buffer		= array();
		$queries	= array();
		$in_string	= false;
	
		// Trim any whitespace.
		$sql = trim($sql);
	
		// Remove comment lines.
		$sql = preg_replace("/\n\#[^\n]*/", '', "\n".$sql);
	
		// Parse the schema file to break up queries.
		for ($i = 0; $i < strlen($sql) - 1; $i ++)
		{
		if ($sql[$i] == ";" && !$in_string) {
		$queries[] = substr($sql, 0, $i);
		$sql = substr($sql, $i +1);
		$i = 0;
		}
	
		if ($in_string && ($sql[$i] == $in_string) && $buffer[1] != "\\") {
		$in_string = false;
		}
		elseif (!$in_string && ($sql[$i] == '"' || $sql[$i] == "'") && (!isset ($buffer[0]) || $buffer[0] != "\\")) {
		$in_string = $sql[$i];
		}
		if (isset ($buffer[1])) {
		$buffer[0] = $buffer[1];
		}
			$buffer[1] = $sql[$i];
		}
	
		// If the is anything left over, add it to the queries.
		if (!empty($sql)) {
		$queries[] = $sql;
	}
		return $queries;
	}
	private function extractArchive($source, $destination)
	{
		// Cleanup path
		$destination= JPath :: clean($destination);
		$source= JPath :: clean($source);
		$result= JArchive :: extract($source, $destination);
		if($result === false)
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	private function oseunpack($src, $dest, $file)
	{
		$extractdir= JPath :: clean($dest);
		$archivename= JPath :: clean($src.DS.$file);
		// do the unpacking of the archive
		$result= JArchive :: extract($archivename, $extractdir);
		if($result === false)
		{
			return false;
		}
		else
		{
			if(JFile :: delete($archivename))
			{
				return true;
			}
		}
	}
	function installMenuPatch()
	{
		return true;
	}
	function installModulePatch()
	{

			return true;
	}
	function installViews($step)
	{
		$html= '';
		$html .= '<div style="width:100px; float:left;">'.JText :: _('No Views to install for this component').'</div>';
		$result= null;
		$db= JFactory :: getDBO();

		$result= $this->installViewsDB();
		$viewhtml= '';

		if($result == true)
		{
			$html .= $this->successStatus;
			$autoSubmit= $this->getAutoSubmitFunction();
			//$form = $this->getInstallForm(5);
			$message= $autoSubmit.$html;
			$status= true;
		}
		else
		{
			$html .= $this->failedStatus;
			$html .= $viewhtml;
			$errorMsg= $this->getErrorMessage($step, $step);
			$message= $html.$errorMsg;
			$status= false;
			$step= $step -1;
		}
		$drawdata= new stdClass();
		$drawdata->message= $message;
		$drawdata->status= true;
		$drawdata->step= $step;
		$drawdata->title= JText :: _('CREATING VIEWS');
		$drawdata->install= 1;
		return $drawdata;
	}
	private function installLanguage($type)
	{
		if ($type =='back')
		{
			$src= JPATH_ADMINISTRATOR.DS.'components'.DS.$this->component.DS.'language'.DS.'en-GB'.DS.'en-GB.'.$this->component.'.ini';
			$dest= JPATH_ADMINISTRATOR.DS.'language'.DS.'en-GB'.DS.'en-GB.'.$this->component.'.ini';
		}
		elseif($type =='backsys')
		{
			$src= JPATH_ADMINISTRATOR.DS.'components'.DS.$this->component.DS.'language'.DS.'en-GB'.DS.'en-GB.'.$this->component.'.sys.ini';
			$dest= JPATH_ADMINISTRATOR.DS.'language'.DS.'en-GB'.DS.'en-GB.'.$this->component.'.sys.ini';
		}
		else
		{
			$src= JPATH_SITE.DS.'components'.DS.$this->component.DS.'language'.DS.'en-GB'.DS.'en-GB.'.$this->component.'.ini';
			$dest= JPATH_SITE.DS.'language'.DS.'en-GB'.DS.'en-GB.'.$this->component.'.ini';
		}
		if(!JFile :: copy($src, $dest))
		{
			echo JText :: _('Unable to copy language file');
			return false;
		}
		else
		{
			return true;
		}
	}
	// Changes from here; 
	private function readSQLFile($filename)
	{
		$file = JPATH_COMPONENT.DS.'sql'.DS.$filename;
		$query = JFile::read($file);
		return $query; 
	}
	
	private function fixIntegrity()
	{
		$db = JFactory::getDBO();
		$exists= $this->checkIndexes('ip32_start', '#__osefirewall_iptable');
		if ($exists==false)
		{
			$query = "ALTER TABLE `#__osefirewall_iptable` ADD INDEX `osefirewall_iptable` ( `ip32_start` ) ";
			$db->setQuery($query); 
			$db->query(); 
		}
		
		$exists= $this->checkIndexes('type', '#__ose_secConfig');
		if ($exists==false)
		{
			$query = "ALTER TABLE `#__ose_secConfig` ADD INDEX `secConfigindex` ( `type` ) ";
			$db->setQuery($query); 
			$db->query(); 
		}
		return true;
	}
	private function checkIndexes($columnname, $table)
	{
		$db = JFactory::getDBO();
		$query = "SHOW INDEX FROM `{$table}`";
		$db->setQuery($query); 
		$indexes = $db->loadObjectlist();
		foreach ($indexes as $index )
		{
			if ($index->Column_name == $columnname)
			{
				return true;
			}
		} 
		return false; 
	}
	private function installSQL()
	{
		
		$db = JFactory::getDBO();
		$data = $this->readSQLFile('createTable.sql');
		$queries = $this->_splitQueries($data);
		foreach ($queries as $query)
		{	
				$db->setQuery($query);
				if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
				}
		}
		
		$query = "SELECT `key` FROM `#__ose_secConfig` WHERE `key` = 'threshold' ";
		$db->setQuery($query);
		$result = $db->loadResult();
		if (empty($result))
		{
			$query = $this->readSQLFile('dataSecConfig.sql');
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}

		$query = "SELECT COUNT(id) FROM `#__ose_app_email` WHERE `app` = 'antihack'";
		$db->setQuery($query);
		$result = $db->loadResult();
		if ($result==0)
		{
			$query = $this->readSQLFile('dataAppEmail.sql');
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		
		$query = "SELECT COUNT(id) FROM `#__osefirewall_signatures` ";
		$db->setQuery($query);
		$result = $db->loadResult();
		if ($result==0)
		{
			$query = $this->readSQLFile('dataSignature.sql');
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		
		$query = "SELECT COUNT(id) FROM `#__osefirewall_signatures_bk` ";
		$db->setQuery($query);
		$result = $db->loadResult();
		if ($result==0)
		{
			$query = $this->readSQLFile('dataSignature.sql');
			$query = str_replace('#__osefirewall_signatures', '#__osefirewall_signatures_bk', $query); 
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		
		$query = "SELECT COUNT(id) FROM `#__osefirewall_filters` ";
		$db->setQuery($query);
		$result = $db->loadResult();
		if ($result==0)
		{
			$query = $this->readSQLFile('dataPatterns.sql');
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		
		$query = "SELECT COUNT(id) FROM `#__osefirewall_filters_bk` ";
		$db->setQuery($query);
		$result = $db->loadResult();
		if ($result==0)
		{
			$query = $this->readSQLFile('dataPatterns.sql');
			$query = str_replace('#__osefirewall_filters', '#__osefirewall_filters_bk', $query);
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		
		$query = "SELECT COUNT(id) FROM `#__osefirewall_attacktype` ";
		$db->setQuery($query);
		$result = $db->loadResult();
		if ($result==0)
		{
			$query = $this->readSQLFile('dataAttacktype.sql');
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		
		return true;
	}
	private function isViewExists($view)
	{
		$db= JFactory :: getDBO();
		if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') 
		{
			$query= "SHOW CREATE VIEW `{$view}`";
		}
		else
		{		
			$query= "SHOW TABLE STATUS LIKE '{$view}'";
		}
		$query = $this->convertQuery($query);
		$db->setQuery($query); 
		$result= $db->loadObjectlist();
		return (!empty($result))?true: false; 
	}
	private function convertQuery($query)
	{
		if (class_exists('SConfig'))
		{
			$config= new SConfig();
		}
		else
		{
			$config= new JConfig();
		}
		$return = str_replace('#__', $config->dbprefix, $query);
		return  $return; 
	}
	private function installViewsDB()
	{
		$db= JFactory :: getDBO();
		$result= $this->isViewExists('#__osefirewall_aclipmap');
		if(empty($result))
		{
			$query = $this->readSQLFile('viewAclipmap.sql');
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		
		$result= $this->isViewExists('#__osefirewall_attackmap');
		if(empty($result))
		{
			$query = $this->readSQLFile('viewAttackmap.sql');
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		
		$result= $this->isViewExists('#__osefirewall_attacktypesum');
		if(empty($result))
		{
			$query = $this->readSQLFile('viewAttackTypesum.sql');
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		
		$result= $this->isViewExists('#__ose_app_adminemailmap');
		if(empty($result))
		{
			$query = $this->readSQLFile('viewAdminEmail.sql');
			$db->setQuery($query);
			if(!$db->query()) {
					echo JText :: _('Unable to create tables');
					echo $db->getErrorMsg();
					return false;
			}
		}
		return true;
	}

	private function clearInstallation($step)
	{
		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');
		$html= '';
		$zip= array();
		$zip[]= $this->backendPath.'admin.zip';
		$zip[]= $this->backendPath.$this->cpuFile;
		$zip[]= $this->backendPath.'com_cpu_admin.zip';
		$zip[]= $this->backendPath.'com_cpu_site.zip';
		$result= true;
		foreach($zip as $z)
		{
			$html .= '<div style="width:500px; float:left;">'.JText :: _('Clearing file').' '.$z.'</div>';
			$result= JFile :: delete($z);
			if($result == true)
			{
				$html .= $this->successStatus;
				$autoSubmit= $this->getAutoSubmitFunction();
				//$form = $this->getInstallForm(5);
				$message= $autoSubmit.$html;
				$status= true;
			}
			else
			{
				$html .= $this->failedStatus;
				$errorMsg= $this->getErrorMessage($step, $step);
				$message= $html.$errorMsg;
				$status= false;
				$step= $step -1;
			}
		}

		$drawdata= new stdClass();
		$drawdata->message= $message;
		$drawdata->status= true;
		$drawdata->step= $step;
		$drawdata->title= JText :: _('CREATING VIEWS');
		$drawdata->install= 1;
		return $drawdata;
	}
	private function installationComplete($step)
	{
		$cache= JFactory :: getCache();
		$cache->clean();
		$version= '';
		$file= dirname(__FILE__).DS.'installer.dummy.ini';
		if(JFile :: exists($file) && JFile :: delete($file))
		{
			$html= '';
			$html .= '<div style="margin: 30px 0; padding: 10px; background: #edffb7; border: solid 1px #8ba638; width: 50%; -moz-border-radius: 5px; -webkit-border-radius: 5px;">
									<h3 style="padding: 0; margin: 0 0 5px;">Installation has been completed</h3></div>';
		}
		else
		{
			$html= '<div></div>';
			$html .= '<div style="margin: 30px 0; padding: 10px; background: #edffb7; border: solid 1px #8ba638; width: 50%; -moz-border-radius: 5px; -webkit-border-radius: 5px;">
									<h3 style="padding: 0; margin: 0 0 5px;">Installation has been completed</h3>However we were unable to remove the file <b>installer.dummy.ini</b> located in the '.dirname(__FILE__).' folder. Please remove it manually in order to completed the installation.</div>';
		}
		ob_start();
?>

		<div style="margin: 30px 0; padding: 10px; background: #fbfbfb; border: solid 1px #ccc; width: 50%; -moz-border-radius: 5px; -webkit-border-radius: 5px;">
			<h3 style="color: red;">IMPORTANT!!</h3>
			<div>Before you begin, you might want to take a look at the following documentations first</div>
			<ul style="background: none;padding: 0; margin-left: 15px;">
				<li style="background: none;padding: 0;margin:0;"><a href="http://www.opensource-excellence.com/documentation/tutorials-ose-anti-hacker/item/411.html" target="_blank">Activating OSE Anti Hacker</a></li>
				<li style="background: none;padding: 0;margin:0;"><a href="http://www.opensource-excellence.com/documentation/tutorials-ose-anti-hacker.html" target="_blank">Configuring OSE Anti Hacker</a></li>
			</ul>
			<div>You can read the full documentation at <a href="http://wiki.opensource-excellence.com" target="_blank">OSE Wiki Website</a></div>
		</div>

	<?php

		$content= ob_get_contents();
		ob_end_clean();
		$html .= $content;
		//$form = $this->getInstallForm(0, 0);
		$message= $html;
		$drawdata= new stdClass();
		$drawdata->message= $message;
		$drawdata->status= true;
		$drawdata->step= $step;
		$drawdata->title= JText :: _('INSTALLATION COMPLETED');
		$drawdata->install= 0;
		return $drawdata;
	}
	private function getErrorMessage($error= "", $extraInfo= "")
	{
		switch($error)
		{
			case 0 :
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('The operation is invalid');
				break;
			case 1 :
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('The file is missing');
				break;
			case 2 :
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('OSE BACKEND EXTRACT FAILED WARN');
				break;
			case 3 :
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('OSE CPU INSTALL FAILED');
				break;
			case 4 :
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('OSE FRONTEND EXTRACT FAILED WARN');
				break;
			case 5 :
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('Error creating OSE tables');
				break;
			case 6 :
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('Error creating OSE tables');
				break;
			case 7 :
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('Error fixing OSE table integrity');
				break;
			case 8 :
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('Error creating Database Views');
				break;
			case 101 :
				$errorWarning= $error.' : '.JText :: _('PHP version is lower than 5.2, your version is'.' '.$extraInfo);
				break;
			default :
				$error=(!empty($error)) ? $error : '99';
				$errorWarning= $error.'-'.$extraInfo.' : '.JText :: _('UNEXPECTED ERROR WARN');
				break;
		}
		ob_start();
?>
		<div style="font-weight: 700; color: red; padding-top:10px">
			<?php echo $errorWarning; ?>
		</div>
		<div id="communityContainer" style="margin-top:10px">
			<div><?php echo JText::_('OSE INSTALLATION ERROR HELP'); ?></div>
			<div><a href="http://wiki.opensource-excellence.com/index.php?title=Trouble_Shooting_-_OSE_Security_3">http://wiki.opensource-excellence.com/index.php?title=Trouble_Shooting_-_OSE_Security_3</a></div>
		</div>
		<?php

		$errorMsg= ob_get_contents();
		@ ob_end_clean();
		return $errorMsg;
	}
}

?>