<?php
/**
 * @version     6.0 +
 * @package       Open Source Excellence Security Suite
 * @subpackage    Open Source Excellence CPU
 * @author        Open Source Excellence {@link http://www.opensource-excellence.com}
 * @author        Created on 30-Sep-2010
 * @author        Updated on 30-Mar-2013 
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 * @copyright Copyright (C) 2008 - 2010- ... Open Source Excellence
 *
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
if (!defined('_OSEEXEC') && !defined('OSE_ADMINPATH')) {
	die('Direct Access Not Allowed');
}
class oseFirewallStat {
	var $osCPU = null; 
	public function __construct() {
		$this->oseCPU = new oseCPU(); 
	}
	public function getAttackSummary()
	{
		return $this->convertAttackSummary($this->getAttackSummaryDB());
	}
	private function getAttackSummaryDB()
	{
		$db = OSEDBO::instance();
		$query = "SELECT COUNT(aclid) as count, DATE(datetime) as date, attacktypeid ".
				 "FROM `#__osefirewall_attacktypesum` ".
				 "WHERE DATEDIFF( NOW(), datetime ) <= 10 AND `status` IN (1,2)  ".
				 "GROUP BY DATE(datetime), attacktypeid ";
		$db->setQuery($query); 
		return $db->loadObjectList();
	}
	private function convertAttackSummary($results)
	{
		$return = array();
		$i=0;  
		foreach ($results as $result)
		{
			if (!isset($return[$i]['date']))
			{
				$return[$i]['date']=$result->date;
			}
			if ($result->date!=$return[$i]['date'])
			{
				$i++;
				$return[$i]['date']=$result->date;
			}
			$result->attacktypeid=(!empty($result->attacktypeid))?$result->attacktypeid:0;
			$return[$i]['type'.$result->attacktypeid]= (int)$result->count;
		}
		return $return;
	}
	private function getAttackTypeArray()
	{
		$results = $this->getAttackTypesDB();
		$return = array(); 
		foreach ($results as $result)
		{
			$return[$result->id]= $result->name;
		}
		return $return;
	}
	private function getAttackTypesDB($ids=array())
	{
		$db = OSEDBO::instance();
		$where = array();
		if (!empty($ids))
		{
			$where[] = "`id` NOT IN (".implode(",", $ids).")";
		}
		$where= $db->implodeWhere($where);
		$query = "SELECT `id`, `name` FROM `#__osefirewall_attacktype` ".$where;
		$db->setQuery($query); 
		return $db->loadObjectList();
	}
	public function getAttackTypes($ids)
	{
		return $this->getAttackTypesDB($ids);
	}
	public function getACLIPMap()
	{
		$limit = $this->oseCPU->getInt('limit', 25);
		$start = $this->oseCPU->getInt('start', 0);
		$page = $this->oseCPU->getInt('page', 1);
		$search = $this->oseCPU->getVar('search', null);
		$status = $this->oseCPU->getInt('status', null);
		$start = $limit * ($page-1);    
		return $this->convertACLIPMap($this->getACLIPMapDB($search, $status, $start, $limit));
	}
	private function getACLIPMapDB($search, $status, $start, $limit)
	{
		$db = OSEDBO::instance();
		$where = array(); 
		if (!empty($search))
		{
			$where[] = "`name` LIKE ".$db->Quote($search.'%', true)." OR `ip32_start` = ".$db->Quote(ip2long($search), true);;
		}
		if (!empty($status))
		{
			$where[] = "`status` = ".(int)$status;
		}
		$where = $db->implodeWhere($where);
		$query = "SELECT * FROM `#__osefirewall_aclipmap` " .$where
				 ." ORDER BY datetime DESC LIMIT ".$start.", ".$limit;
		$db->setQuery($query); 
		$results = $db->loadObjectList();
		return $results; 
	}
	private function convertACLIPMap($results)
	{
		$i = 0;
		foreach ($results as $result)
		{
			if (empty($results[$i]->country_code))
			{
				$results[$i]->country_code = $this->updateCountryCode($results[$i]->id, $results[$i]->ip32_start);
			}
			$results[$i]->country_code = $this->getCountryImage($results[$i]->country_code);
			$results[$i]->ip32_start= long2ip((float)$results[$i]->ip32_start);
			$results[$i]->ip32_end= long2ip((float)$results[$i]->ip32_end);
			if (empty($results[$i]->host))
			{
				//$results[$i]->host = $this->updateIPHost($results[$i]->id, $results[$i]->ip32_start);
			}
			$results[$i]->view = $this->getViewIcon($results[$i]->id);
			$results[$i]->status = $this->getStatusIcon($results[$i]->id, $results[$i]->status);
			$i++;
		}
		return  $results;
	}
	private function getViewIcon($id)
	{
		return "<a href='#' onClick= 'viewIPdetail(".urlencode($id).")' ><img src='components/com_ose_antihacker/assets/images/page_white_magnify.png' /></a>"; 
	}
	private function getStatusIcon($id, $status)
	{
		switch($status) {
                   	case '3':
                   		return "<a href='#' onClick= 'changeItemStatus(".urlencode($id).", 2)' ><div class='ose-grid-accept'></div></a>";
                   	break;
                   	case '2': 
                   		return "<a href='#' onClick= 'changeItemStatus(".urlencode($id).", 1)' ><div class='ose-grid-error'></div></a>";
                   	break;
                   	case '1': 
                   		return "<a href='#' onClick= 'changeItemStatus(".urlencode($id).", 3)' ><div class='ose-grid-delete'></div></a>";
                   	break;
                   	default: 
                   	 	return '';
                   	break;	
        }
	}
	public function updateHost($acl_id)
	{
		$aclinfo = $this->getACLIPMapByIDDB($acl_id);
		$ip_start = long2ip($aclinfo->ip32_start); 
		$result = $this->updateIPHost($acl_id, $ip_start);
		if ($result ==false)
		{
			return false; 
		}
		return true; 
	}	
	private function updateIPHost($acl_id, $ip_start)
	{
		$host = $this->get_host($ip_start);
		$db = OSEDBO::instance();
		$query = " UPDATE `#__osefirewall_acl` SET `host` = ". $db->Quote($host, true).
				 " WHERE `id` = ". (int)$acl_id;
		$db->setQuery($query);
		$result = $db->query() ;
		if ($result == true)
		{
			return $host;
		}
		else
		{
			return false;
		}
	}
	private function get_host($ip){
		if (empty($ip))
		{
			return 'N/A';
		}
        $ptr= implode(".",array_reverse(explode(".",$ip))).".in-addr.arpa";
        $host = dns_get_record($ptr,DNS_PTR);
        if ($host == null) 
        {
        	return $ip;
        }
        else 
        {
        	return $host[0]['target'];
        }
	}
	private function getCountryImage($country_code)
	{
		if (empty($country_code))
		{
			return '';
		}
		else
		{
			return "<img src='components/com_ose_antihacker/assets/images/flags/".strtolower($country_code).".png' alt='".$country_code."' />";
		}	
	}
	private function getCountryCodebyIP($ip32_start)
	{
		$db = OSEDBO::instance();
		$ip32_start = substr("0000000000" . sprintf('%u',$ip32_start), -10);
		$query = " SELECT `country_code` FROM `#__ose_app_geoip` ".
				 " WHERE `ip32_start`<= ".$db->Quote($ip32_start)." AND ".$db->Quote($ip32_start)." <= `ip32_end`;";
		$db->setQuery($query);
		$country = $db->loadResult();
		$country  = strtolower($country); 
		return $country;  
	}
	private function updateCountryCode($acl_id, $ip32_start)
	{
		$db = OSEDBO::instance();
	 	$country_code = $this->getCountryCodebyIP($ip32_start);
	 	$varValues = array (
				'country_code' => $country_code
		);
		$result = $db->addData('update', '#__osefirewall_acl', 'id', (int)$acl_id, $varValues);
		if ($result == true)
		{
			return $country_code; 
		}
		else
		{
			return false; 
		}
	}
	public function getACLIPTotal()
	{
		$db = OSEDBO::instance();
		return $db->getTotalNumber('id', '#__osefirewall_aclipmap'); 
	}
	public function remvoeACLRule($aclid)
	{
		$ids = $this->getIDSOnACLID($aclid);
		if (!empty($ids['detattacktype_id']))
		{
			$result = $this->deleteAttackTypeID($aclid, $ids['detattacktype_id']);
			if 	($result==false)
			{
				return false;
			}
		}
		if (!empty($ids['ipid']))
		{
			$result = $this->deleteIPID($aclid, $ids['ipid']);
			if 	($result==false)
			{
				return false;
			}
		} 
		if (!empty($ids['aclid']))
		{
			$result = $this->deleteACLID($aclid);
			if 	($result==false)
			{
				return false;
			}
		} 
		return true; 
	}
	private function deleteACLID($aclid)
	{
		$db = OSEDBO::instance();
		return $db->deleteRecord(array('id'=>$aclid), '#__osefirewall_acl');
	}
	private function deleteIPID($aclid, $ipid)
	{
		$db = OSEDBO::instance();
		return $db->deleteRecord(array('id'=>$ipid, 'acl_id'=>$aclid), '#__osefirewall_iptable');
	}
	private function deleteAttackTypeID($aclid, $ids)
	{
		foreach ($ids as $detattacktype_id)
		{
			$result = $this->deleteAttackTypeIDDB($aclid, $detattacktype_id);
			if ($result == false)
			{
				return false;
				break;
			}
		}
		return true;
	}
	private function deleteAttackTypeIDDB($aclid, $detattacktype_id)
	{
		$db = OSEDBO::instance();
		$result = $db->deleteRecord(array('acl_id'=>$aclid, 'detattacktype_id'=>$detattacktype_id), '#__osefirewall_detected');	
		if ($result == true)
		{
			$result = $db->deleteRecord(array('detattacktype_id'=>$detattacktype_id), '#__osefirewall_detcontdetail');
			if ($result == true)
			{
				$result = $db->deleteRecord(array('id'=>$detattacktype_id), '#__osefirewall_detattacktype');	
			}
		}
		return $result; 	
	}
	private function getIDSOnACLID($aclid)
	{
		$return = array(); 
		$return['aclid']= $aclid;
		$return['ipid'] = $this->getIPIDOnAclidDB($aclid);
		$return['detattacktype_id']=$this->getDetAttackIDOnAclidDB($aclid);
		return $return;  
	} 
	private function getIPIDOnAclidDB($aclid)
	{
		$db = OSEDBO::instance();
		$query = "SELECT `ipid` FROM `#__osefirewall_aclipmap` WHERE `id` = ". (int)$aclid;
		$db->setQuery($query);
		$result = $db->loadResult();
		return $result;
	}
	private function getDetAttackIDOnAclidDB($aclid)
	{
		$db = OSEDBO::instance();
		$query = "SELECT `detattacktype_id` FROM `#__osefirewall_detected` WHERE `acl_id` = ". (int)$aclid;
		$db->setQuery($query);
		$results = $db->loadResultArray();
		return $results;
	}
	public function changeACLStatus($aclid, $status)
	{
		$db = OSEDBO::instance();
		$varValues = array (
				'status' => (int)$status
		);
		$result = $db->addData('update', '#__osefirewall_acl', 'id', (int)$aclid, $varValues);
		return $result;
	}
	public function getAttackDetail($aclid)
	{
		
		$aclrule = $this->getACLIPMapByIDDB($aclid);
		$html = "<table width='100%' class='stat'>"; 
		if (empty($aclrule))
		{
			$html .= "<tr><td class='label'>".JText::_('Result')."</td><td class='attackcontent'>".JText::_("No attack information found")."</td></tr>";
		}
		else
		{
			$html .= "<tr><td class='label'><div>".JText::_('IP Access Rule ID')."</div></td><td class='attackcontent'>".$aclrule->id."</td></tr>";
			$html .= "<tr><td class='label'><div>".JText::_('Country')."</div></td><td class='attackcontent'>".$this->getCountryImage($aclrule->country_code)."</td></tr>";
			$html .= "<tr><td class='label'><div>".JText::_('Logged Time')."</div></td><td class='attackcontent'>".$aclrule->datetime."</td></tr>";
			$html .= "<tr><td class='label'><div>".JText::_('Referer')."</div></td><td class='attackcontent'>".$this->getRefererByIDDB($aclrule->referers_id) ."</td></tr>";
			$html .= "<tr><td class='label'><div>".JText::_('Target')."</div></td><td class='attackcontent'>".$this->getPageByIDDB($aclrule->pages_id) ."</td></tr>";
		
			$aclattackmap = $this->getACLAttackMapByIDDB($aclid);
			if (!empty($aclattackmap))
			{
				$tmp= null;
				
				foreach ($aclattackmap as $item)
				{
					
					if (!isset($detcontent_id) || $item->attacktypeid==1 || $item->attacktypeid==11)
					{
						$detcontent_id = $item->detcontent_id;
					}
					if (!isset($tag) || $item->attacktypeid==1 || $item->attacktypeid==11)
					{
						$tag = $item->tag;
					}
					if (!isset($content) || $item->attacktypeid==1 || $item->attacktypeid==11)
					{
						$content = $item->content;
					}
					if ($item->attacktypeid==1 || $item->attacktypeid==11 || ($detcontent_id !=$item->detcontent_id && $tag != $item->tag && $content != $item->content))
					{
						$html .="<tr>";
						if ($item->attacktypeid==1 || $item->attacktypeid==11)
						{
							$detcontent_id ==$item->detcontent_id;
							$html .= "<td class='label'><div>".$item->name."</div></td>";
						}
						else
						{
							$html .= "<td class='label'><div>".$item->keyname."</div></td>";
						}
						
						$detcontent_id = $item->detcontent_id;
						$tag = $item->tag;
						$content = $item->content;
						$html .= "<td class='attackcontent'>";
						$html .= "Detected Content: <span style='color:red;'>".$item->content."</span>";
					}
					else if ($detcontent_id ==$item->detcontent_id && ($item->attacktypeid!=1 && $item->attacktypeid!=11) && ($tag != $item->tag))
					{
						$html .= ', '.$item->name;
					}
					else if ($detcontent_id ==$item->detcontent_id && ($item->attacktypeid!=1 && $item->attacktypeid!=11))
					{
						$html .= "<tr><td class='label'><div>AttackType: </div></td><td>".$item->name."</td>";
					}				
					else
					{
						$html .= "</td>";
						$html .= "</tr>";
					}
				}
				
			}			
		}
		$html .= "</table>";
		return $html; 
	}
	private function getACLAttackMapByIDDB($id)
	{
		$db = OSEDBO::instance();
		$query = "SELECT * FROM `#__osefirewall_attackmap` WHERE `aclid` = ". (int)$id.
				 " ORDER BY `var_id` ASC, `detcontent_id` ASC"; 
		$db->setQuery($query);
		$results = $db->loadObjectList();
		return $results;
	}
	private function getACLIPMapByIDDB($id)
	{
		$db = OSEDBO::instance();
		$query = "SELECT * FROM `#__osefirewall_aclipmap` WHERE `id` = ". (int)$id; 
		$db->setQuery($query);
		$result = $db->loadObject();
		return $result; 
	}
	private function getRefererByIDDB($id)
	{
		$db = OSEDBO::instance();
		$query = "SELECT `referer_url` FROM `#__osefirewall_referers` WHERE `id` =".(int)$id;
		$db->setQuery($query);
		return $db->loadResult();  
	}
	private function getPageByIDDB($id)
	{
		$db = OSEDBO::instance();
		$query = "SELECT `page_url` FROM `#__osefirewall_pages` WHERE `id` =".(int)$id;
		$db->setQuery($query);
		return $db->loadResult();  
	}
	public function getSignatures()
	{
		$limit = $this->oseCPU->getInt('limit', 25);
		$start = $this->oseCPU->getInt('start', 0);
		$page = $this->oseCPU->getInt('page', 1);
		$search = $this->oseCPU->getVar('search', null);
		$status = $this->oseCPU->getInt('status', null);
		$start = $limit * ($page-1);  
		return $this->convertSignatures($this->getSiganturesDB($search, $status, $start, $limit));
	}
	private function getSiganturesDB($search, $status, $start, $limit)
	{
		$db = OSEDBO::instance();
		$where = array(); 
		if (!empty($search))
		{
			$where[] = "`signature` LIKE ".$db->Quote($search.'%', true);
		}
		if ($status===1 || $status===0)
		{
			$where[] = "`action` = ".(int)$status;
		}
		$where = $db->implodeWhere($where);
		$query = "SELECT * FROM `#__osefirewall_signatures`".$where
				 ." ORDER BY id ASC LIMIT ".$start.", ".$limit;
		$db->setQuery($query);
		return $db->loadObjectList();
	}
	private function convertSignatures($results)
	{
		$i = 0;
		foreach ($results as $result)
		{
			$results[$i]->signature= htmlspecialchars($results[$i]->signature);
			$results[$i]->action = $this->getActionIcon($results[$i]->id, $results[$i]->action);
			$i ++; 
		}
		return $results; 
	}
	private function getActionIcon($id, $status)
	{
		switch($status) {
                   	case '0': 
                   		return "<a href='#' onClick= 'changeItemStatus(".urlencode($id).", 1)' ><div class='ose-grid-delete'></div></a>";
                   	break;
                   	case '1': 
                   		return "<a href='#' onClick= 'changeItemStatus(".urlencode($id).", 0)' ><div class='ose-grid-accept'></div></a>";
                   	break;
                   	default: 
                   	 	return '';
                   	break;	
        }
	}
	public function getSignaturesTotal()
	{
		$db = OSEDBO::instance();
		return $db->getTotalNumber('id', '#__osefirewall_signatures');
	}
	public function changeL1RuleStatus($id, $status)
	{
		$db = OSEDBO::instance();
		$varValues = array (
				'action' => (int)$status
		);
		return $db->addData('update', '#__osefirewall_signatures', 'id', (int)$id, $varValues);
	}
	public function addsignature($signature, $status)
	{
		$db = OSEDBO::instance();
		$varValues = array(
					'id' => 'DEFAULT',
					'signature' => $signature,
					'action' => (int)$status,
					'attacktype' => '["1"]'
				);
		$id = $db->addData ('insert', '#__osefirewall_signatures', '', '', $varValues);
		return $id; 
	}
	private function getDetattacktypeIDByRuleID($rule_id, $attacktypeidArray)
	{
		$db = OSEDBO::instance();
		$attacktypeids = '('.implode(',', $attacktypeidArray).')';   
		$query = "SELECT `detattacktype_id` FROM `#__osefirewall_attackmap` WHERE `rule_id` =".(int)$rule_id. " AND `attacktypeid` IN ". $attacktypeids; 
		$db ->setQuery($query);
		return $db->loadResultArray(); 
	}
	private function getDetattacktypeIDByVarID($var_id)
	{
		$db = OSEDBO::instance();
		$query = "SELECT `detattacktype_id` FROM `#__osefirewall_attackmap` WHERE `var_id` =".(int)$var_id; 
		$db ->setQuery($query);
		return $db->loadResultArray(); 
	}
	public function deletesignature($id)
	{
		$detattacktype_ids = $this->getDetattacktypeIDByRuleID($id, array(1));
		$result = $this->deleteDectectedAttacks($detattacktype_ids);	
		if ($result==true)
		{	
			$result = $this->deleteSignaturebyID($id);
		}
		return $result; 
	}
	
	private function deleteAttackTypeDetailByID($detattacktype_id)
	{
		$db = OSEDBO::instance();
		return $db->deleteRecord(array('detattacktype_id'=>$detattacktype_id), '#__osefirewall_detcontdetail');
	}
	private function deleteAttackTypebyID($detattacktype_id)
	{
		$db = OSEDBO::instance();
		return $db->deleteRecord(array('id'=>$detattacktype_id), '#__osefirewall_detattacktype');
	}
	private function deleteSignaturebyID($id)
	{
		$db = OSEDBO::instance();
		return $db->deleteRecord(array('id'=>$id), '#__osefirewall_signatures');
	}
	public function restoreRules($type)
	{
		switch($type)
		{
			case 'signature':
				$this->restoreSignatureDatabase();
			break;
			case 'filters':
				$this->restoreFilterDatabase();
			break;
		}
	}
	private function restoreSignatureDatabase()
	{
		$result = $this->cleanSignatureDatabase('signature');
		if ($result ==true)
		{
			$db = OSEDBO::instance();
			$query = "INSERT INTO `#__osefirewall_signatures` SELECT * FROM `#__osefirewall_signatures_bk`";
			$db->setQuery($query); 
			return $db->query();
		}
		else
		{
			return false; 
		}
	}
	private function restoreFilterDatabase()
	{
		$result = $this->cleanSignatureDatabase('filter');
		if ($result ==true)
		{
			$db = OSEDBO::instance();
			$query = "INSERT INTO `#__osefirewall_filters` SELECT * FROM `#__osefirewall_filters_bk`";
			$db->setQuery($query); 
			return $db->query();
		}
		else
		{
			return false; 
		}
	}
	private function cleanSignatureDatabase($type)
	{
		$db = OSEDBO::instance();
		if ($type=='signature')
		{
			$query = "TRUNCATE `#__osefirewall_signatures`";
		}
		elseif ($type=='filter')
		{
			$query = "TRUNCATE `#__osefirewall_filters`";
		}
		$db->setQuery($query); 
		return $db->query();
	}
	public function getRulesets()
	{
		$limit = $this->oseCPU->getInt('limit', 25);
		$start = $this->oseCPU->getInt('start', 0);
		$page = $this->oseCPU->getInt('page', 1);
		$search = $this->oseCPU->getVar('search', null);
		$status = $this->oseCPU->getInt('status', null);
		$start = $limit * ($page-1);  
		return $this->convertRulesets($this->getRulesetsDB($search, $status, $start, $limit));
	}
	private function getRulesetsDB($search, $status, $start, $limit)
	{
		$db = OSEDBO::instance();
		$where = array(); 
		if (!empty($search))
		{
			$where[] = "`description` LIKE ".$db->Quote('%'.$search.'%', true);
		}
		if ($status===1 || $status===0)
		{
			$where[] = "`action` = ".(int)$status;
		}
		$where = $db->implodeWhere($where);
		$query = "SELECT `id`,`action`,`attacktype`,`impact`,`description` FROM `#__osefirewall_filters`".$where
				 ." ORDER BY id ASC LIMIT ".$start.", ".$limit;
		$db->setQuery($query);
		$results = $db->loadObjectList(); 
		return $results;
	}
	private function convertRulesets($results)
	{
		$i = 0;
		$attacktypes = $this->getAttackTypeArray();
		foreach ($results as $result)
		{
			$results[$i]->description=ucfirst($results[$i]->description); 
			$results[$i]->action = $this->getActionIcon($results[$i]->id, $results[$i]->action);
			$results[$i]->attacktype = $this->attackTypeDecode($attacktypes, $results[$i]->attacktype);
			$i ++; 
		}
		return $results; 
	}
	public function getRulesetsTotal()
	{
		$db = OSEDBO::instance();
		return $db->getTotalNumber('id', '#__osefirewall_filters');
	}
	// $attacktype reads as 1,2,3,4,5; return as [1,2,3,4,5]  
	private function attackTypeEncode($attacktype)
	{
		$attacktype = explode(',', $attacktype);
		$i = 0 ;
		foreach($attacktype as $val)
		{
			$attacktype[$i]= (int)$val;
			$i++; 
		} 
		return oseJSON::encode($attacktype);
	}
	private function attackTypeDecode($attacktypes, $attackids)
	{
		// $attacktype reads as [1,2,3,4,5]; return as Rule1,Rule2,Rule3,Rule4,Rule5 
		$attackids = oseJSON::decode($attackids); 
		$return = array();
		foreach ($attackids as $attackid)
		{
			$return[]= $attacktypes[(int)$attackid];
		}
		$return = implode(", ", $return); 
		return $return;	
	}
	public function changeL2RuleStatus($id, $status)
	{
		$db = OSEDBO::instance();
		$varValues = array (
				'action' => (int)$status
		);
		return $db->addData('update', '#__osefirewall_filters', 'id', (int)$id, $varValues);
		
	}
	public function addruleset($filter, $status, $impact, $description, $attacktype)
	{
		$db = OSEDBO::instance();
		$varValues = array(
					'id' => 'DEFAULT',
					'filter' => $filter,
					'action' => (int)$status,
					'attacktype' => $this->attackTypeEncode($attacktype),
					'impact' => (int)$impact,
					'description' => (string)$description
				);
		$id = $db->addData ('insert', '#__osefirewall_filters', '', '', $varValues);
		return $id; 
	}
	public function deleteruleset($id)
	{
		$detattacktype_ids = $this->getDetattacktypeIDByRuleID($id, array(2,3,4,5,6,7,8,9,10,11,12));
		$result = $this->deleteDectectedAttacks($detattacktype_ids);	
		if ($result==true)
		{
			$result = $this->deleteFilterbyID($id);
		}
		return $result; 
	}
	private function deleteDectectedAttacks($detattacktype_ids)
	{
		foreach ($detattacktype_ids as $detattacktype_id)
		{ 
			$result = $this->deleteAttackTypeDetailByID($detattacktype_id);
			if ($result==false)
			{
				return false;
			}
			$result = $this->deleteDetectedByID($detattacktype_id);
			if ($result==false)
			{
				return false;
			}
			$result = $this->deleteAttackTypebyID($detattacktype_id);
			if ($result==false)
			{
				return false;
			}
		}
		return true; 
	}
	private function deleteDetectedByID($detattacktype_id)
	{
		$db = OSEDBO::instance();
		return $db->deleteRecord(array('detattacktype_id'=>$detattacktype_id), '#__osefirewall_detected');
	}
	private function deleteFilterbyID($id)
	{
		$db = OSEDBO::instance();
		return $db->deleteRecord(array('id'=>$id), '#__osefirewall_filters');
	}
	public function getVariables()
	{
		$limit = $this->oseCPU->getInt('limit', 25);
		$start = $this->oseCPU->getInt('start', 0);
		$page = $this->oseCPU->getInt('page', 1);
		$search = $this->oseCPU->getVar('search', null);
		$status = $this->oseCPU->getInt('status', null);
		$start = $limit * ($page-1);  
		return $this->convertVariables($this->getVariablesDB($search, $status, $start, $limit));
	}
	private function getVariablesDB($search, $status, $start, $limit)
	{
		$db = OSEDBO::instance();
		$where = array(); 
		if (!empty($search))
		{
			$where[] = "`keyname` LIKE ".$db->Quote('%'.$search.'%', true);
		}
		if ($status===1 || $status===2 || $status===3)
		{
			$where[] = "`status` = ".(int)$status;
		}
		if (!empty($limit))
		{
			$limit = "ASC LIMIT ".$start.", ".$limit;
		}
		$where = $db->implodeWhere($where);
		$query = "SELECT * FROM `#__osefirewall_vars`".$where
				 ." ORDER BY id ".$limit;
		$db->setQuery($query);
		$results = $db->loadObjectList(); 
		return $results; 
	}
	private function convertVariables($results)
	{
		$i = 0;
		foreach ($results as $result)
		{
			switch ($results[$i]->status)
			{
				case 1:
					$status = 'Actively scanned';
				break;
				case 2:
					$status = 'Actively filtered';
				break;
				case 3:
					$status = 'Ignored / whitelisted';
				break;
			}
			$results[$i]->status = $this->getStatusIcon($results[$i]->id, $results[$i]->status);
			$results[$i]->statusexp = $status;
			$i ++; 
		}
		return $results; 
	}
	public function getVariablesTotal()
	{
		$db = OSEDBO::instance();
		return $db->getTotalNumber('id', '#__osefirewall_vars');
	}
	public function changeVarStatus($id, $status)
	{
		$db = OSEDBO::instance();
		$varValues = array (
				'status' => (int)$status
		);
		return $db->addData('update', '#__osefirewall_vars', 'id', (int)$id, $varValues);
	}
	public function addvariables($variable, $status)
	{
		$varObject = $this->getVariablebyName($variable);
		if (empty($varObject))
		{
			$db = OSEDBO::instance();
			$varValues = array(
						'id' => 'DEFAULT',
						'keyname' => $variable,
						'status' => (int)$status
					);
			$id = $db->addData ('insert', '#__osefirewall_vars', '', '', $varValues);
			return $id;
		}
		else
		{
			return $varObject->id;
		}
	}
	private function getVariablebyName($variable)
	{
		$db = OSEDBO::instance();
		$query = "SELECT * FROM `#__osefirewall_vars`".
				 " WHERE `keyname` = ".$db->Quote($variable);
		$db->setQuery($query);
		$results = $db->loadObject(); 
		return $results; 
	}	
	public function deletevariable($id)
	{
		$detattacktype_ids = $this->getDetattacktypeIDByVarID($id);
		$result = $this->deleteDectectedAttacks($detattacktype_ids);	
		if ($result==true)
		{
			$result = $this->deleteVariablebyID($id);
		}	
		return $result; 
	}
	private function deleteVariablebyID($id)
	{
		$db = OSEDBO::instance();
		return $db->deleteRecord(array('id'=>$id), '#__osefirewall_vars');
	}
	public function loadDefaultRules($type)
	{
		switch($type)
		{
			case 'joomla':
				$keys = $this->loadJoomlaKeys();
			break;
			case 'wordpress':
				$keys = $this->loadWordpressKeys();
			break;
			case 'jomsocial':
				$keys = $this->loadJSocialKeys();
			break;
		}
		foreach ($keys as $variable)
		{
			$result = $this->addvariables($variable, 3);
			if ($result==false)
			{
				return false;
			}
		}
		return true; 
	}
	private function loadJoomlaKeys()
	{
		$keys = array(); 
		$keys[] = 'COOKIE.CFCLIENT_AVON';
		$keys[] = 'COOKIE.CFCLIENT_LAUSANNE';
		$keys[] = 'COOKIE.CFCLIENT_CFGLOBALS';
		$keys[] = 'COOKIE.omp__super_properties';
		$keys[] = 'COOKIE._okbk';
		$keys[] = 'COOKIE.__utmz';
		$keys[] = 'POST.install_url';
		$keys[] = 'POST.json';
		$keys[] = 'POST.text';
		$keys[] = 'POST.text_mail_new_registration_registrant';
		$keys[] = 'POST.install_directory';
		$keys[] = 'POST.cfg_reg_first_visit_url';
		$keys[] = 'POST.cfg_reg_pend_appr_msg';
		$keys[] = 'POST.cfg_reg_welcome_msg';
		$keys[] = 'POST.filterfieldlist';
		$keys[] = 'POST.params';
		$keys[] = 'POST.sortfields';
		$keys[] = 'POST.title';
		$keys[] = 'POST.url';
		return $keys;
	}
	private function loadWordpressKeys()
	{
		$keys = array(); 
		$keys[] = 'COOKIE.CFCLIENT_AVON';
		$keys[] = 'COOKIE.CFCLIENT_LAUSANNE';
		$keys[] = 'COOKIE.CFCLIENT_CFGLOBALS';
		$keys[] = 'COOKIE.omp__super_properties';
		$keys[] = 'COOKIE._okbk';
		$keys[] = 'COOKIE.__utmz';
		$keys[] = 'POST.siteurl';
		$keys[] = 'POST.redirect_to';
		$keys[] = 'POST.return_to';
		$keys[] = 'POST.home';
		$keys[] = 'POST.current_url';
		$keys[] = 'POST.referredby';
		$keys[] = 'POST._wp_original_http_referer';
		$keys[] = 'POST.link';
		$keys[] = 'POST._jd_wp_twitter';
		return $keys;
	}
	private function loadJSocialKeys()
	{
		$keys = array(); 
		$keys[] = 'POST.arg0';
		$keys[] = 'POST.arg1';
		$keys[] = 'POST.arg2';
		$keys[] = 'POST.arg3';
		$keys[] = 'POST.arg4';
		$keys[] = 'POST.arg5';
		$keys[] = 'POST.arg6';
		$keys[] = 'POST.arg7';
		$keys[] = 'POST.arg8';
		$keys[] = 'POST.arg9';
		$keys[] = 'POST.arg10';
		$keys[] = 'POST.rows';
		return $keys;
	}
	public function clearvariables()
	{
		$variables = $this->getVariablesDB('', '', 0, '');
		foreach ($variables as $variable)
		{
			$result = $this->deletevariable($variable->id);
			if ($result==false)
			{
				return false;
			}
		}
		return true;
	}
	public function getConfiguration($type)
	{
		$return = array(); 
		$db = OSEDBO::instance();
		$query = "SELECT `key`, `value` FROM `#__ose_secConfig` WHERE `type` = ". $db->Quote($type); 
		$db->setQuery($query); 
		$results = $db->loadObjectList();
		foreach ($results as $result)
		{
			if ($type == 'l2var')
			{
				$return['data'][$result->key] = (int)$result->value;
			}
			else
			{
				$return['data'][$result->key] =  $this->convertValue($result->key , $result->value);
			}
		}
		$return['success'] = true; 
		return $return;  
	}
	private function convertValue($key, $value)
	{
		$intarray = array('scanGoogleBots', 'scanMsnBots', 'scanYahooBots', 'blockIP', 'threshold', 'scanFileVirus', 'slient_max_att', 'devMode', 
						  'banpagetype', 'maxvisits', 'decduration', 'antiflooding', 'sfspam', 'sfs_confidence', 
						  'scankey', 'scankeyLength', 'scancookies', 'convertFromSQLKeywords', 'runCentrifuge'); 
		if (in_array($key, $intarray))
		{
			return (int)$value; 
		}
		else
		{
			$value = (!empty($value))?$value:'';
			return $value; 
		}
	}
	public function saveConfiguration($type, $data)
	{
		foreach ($data as $key => $value)
		{
			$keyID = $this->isKeyExistsConf($key, $type);
			if (empty($keyID))
			{
				$result = $this->ConfVariableInsert($key, $value, $type);
			}
			else
			{
				$result = $this->ConfVariableUpdate($keyID, $value);
			}
			if ($result == false)
			{
				return false;
			}
		}
		return true;
	}
	private function isKeyExistsConf($key, $type)
	{
		$db = OSEDBO::instance();
		$query = "SELECT `id` FROM `#__ose_secConfig` WHERE `key` = ". $db->Quote($key, true). " AND `type` = ". $db->Quote($type, true); 
		$db->setQuery($query); 
		return $db->loadResult();
	}
	private function ConfVariableInsert($key, $value, $type)
	{
		$db = OSEDBO::instance();
		$varValues = array(
						'id' => 'DEFAULT',
						'key' => $key,
						'value' => $value, 
						'type' => $type
					);
		$id = $db->addData ('insert', '#__ose_secConfig', '', '', $varValues);
		return $id;
	}
	private function ConfVariableUpdate($keyID, $value)
	{
		$db = OSEDBO::instance();
		$varValues = array (
				'value' => $value
		);
		return $db->addData('update', '#__ose_secConfig', 'id', (int)$keyID, $varValues);
	}
}